#! /usr/bin/env python

## GUISearch:  Browse/Search Code of Applications written using a GUI Framework
## Copyright (C) 2001 Amir Michail

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version. 

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



## need to use module names that are different from those
## in library (and which you are importing); otherwise, get
## AttributeError

import os
import re
import walk
import string
import anydbm
import pickle
import stemmer
import parse_rc
import sys
import kde
import cpp
import misc

## max distance computed in call graph
## larger values may take a long time
MAX_DISTANCE_COMPUTED = 2 

## some maps we need
## we write to memory first as that is faster
## than writing directly to databases on disk


####### GLOBAL, for all apps combined

wm_mem = {}  # word messages
wf_mem = {}  # word functions
mf_mem = {} # message functions
fm_mem = {} # function messages
ms_mem = {} # message source
fs_mem = {} # function source

## bugs:
###  submenus not handled (try konsole)
###  multiple rc's per app, internal names shared among them
###  try searching for icon in konqueror; lots of shared internal names
###  confuse things

porter = stemmer.PorterStemmer()
p = re.compile( r"[\W_]+")  # don't split on digits

### extract profile from function body
### perhaps we should stem here as well?
### code is english after all, so we can stem here?

wordpart_cache = {}

def extract_profile(body):

  global porter
  global p
  global wordpart_cache

  word_list = p.split( body )
  wlist = []
      
  for w in word_list:
    if w != "":
      # look for changes in case to split this up
      # further

      # simple rule:  each capital letter indicates start of new word
      # doesn't handle things like SIGNAL

      ## special cases

      ## HELLO => hello
      ## IOException => io expection

      


      if wordpart_cache.has_key(w):
        wordparts = wordpart_cache[w]
#        print w, " => ", wordparts
      else:
        wordparts = []
        w2 = ""
        for c in w:
          #print "c = " + c
          #print "w2 = " + w2
          #print wlist
          if c >= '0' and c <= '9':
            if w2 != "":
              w2 = w2 + c  # can always extend with digits
          elif c >= 'A' and c <= 'Z':
            if w2 != "" and (w2[len(w2)-1] < "A" or w2[len(w2)-1] > "Z"):
              wordparts.append(string.lower(w2))
              w2 = ""                       
            w2 = w2 + c
          else:
            if len(w2) > 1 and w2[len(w2)-1] >= 'A' and w2[len(w2)-1] <= 'Z':
              t = w2[0:len(w2)-1]
              wordparts.append(string.lower(t))
              w2 = w2[len(w2)-1:]
            w2 = w2 + c
        if w2 != "":
          wordparts.append(string.lower(w2))
        wordpart_cache[w] = wordparts

      wlist = wlist + wordparts

  stemmed_list = []
  for w in wlist:
    if len(w) > 1:
      w2 = porter.stem(w)  # leaves words with numbers at the end alone (e.g., pop3)
      assert( len(w2) >= 1 )
      stemmed_list.append(w2)
  ##print body
  ##print stemmed_list
  return stemmed_list




## we want to propagate GUI message as well
##
## basically, this should come from the closest root
##
## 
def compute_distances( function_parents, is_callback, function_callbacks, function_depth, callback_message ):

  global MAX_DISTANCE_COMPUTED

#  print "...distance preprocessing"

  #### some preprocessing
  for f in all_functions.keys():
    function_depth[f] = {}
    function_callbacks[f] = []
        
  ## some slots are not defined in the code (like closeAllWindows)
  ## and we need to handle them separately
  for c in is_callback.keys():
    if is_callback[c] == 1: # c has itself as a callback
      function_depth[c] = {}
      function_depth[c][c] = 0
      function_callbacks[c] = []
      function_callbacks[c].append(c)
    if not callback_message.has_key(c):
      callback_message[c] = "qevent"


  print "*** computing distances"

  for i in range(MAX_DISTANCE_COMPUTED): 

#    print ".......pass " , i

    ## each function of the form app:function()
    ## each function gets associated with a GUI message (e.g., Open As)
    ## there is all a depth map
    
    for child in function_parents.keys():
      parents = function_parents[child]
      for parent in parents:

        ##### let's look at each callback of the parent and see if it helps us

        for callback in function_callbacks[parent]:
          if function_depth[parent][callback]+1 <= MAX_DISTANCE_COMPUTED:
            if not (callback in function_callbacks[child]):
              function_callbacks[child].append(callback)            
              function_depth[child][callback] = function_depth[parent][callback] + 1
            else:
              # we already know about this callback
              if function_depth[child][callback] > function_depth[parent][callback]+1:
                function_depth[child][callback] = function_depth[parent][callback]+1


            
def add_callback( d, word,  key ):
      if d.has_key(word):
        d[word] = d[word] + "\001" + key # \001 separates different entries
      else:
        d[word] = key # \000 separates fields in a single entry  

### adds support for:
### in:all
### in:package
### in:app
### in:package/app
###
def add_profile(key, profile, d):

    ## pick off package & app
    ##package = key[0:key.index("/")]
    ##app = key[key.index("/")+1:key.index("'")]

    parts = key[0:key.index("'")].split("/")
    # take out empty strings
    parts2 = []
    for p in parts:
      if p != '':
        parts2.append(p) 
    ##print parts2

    for k in range(1, len(parts2)+1):
      ##print "k = ", k
      for i in range(0, len(parts2)-k+1):
        ##print "i = ", i
        piece = parts2[i:i+k]      
        ##print "k = ", k, " and i = ", i, " => " , piece
        piece_str = ""
        for p in piece:
          if piece_str != "":
            piece_str = piece_str + "/"
          piece_str = piece_str + p
        ##print piece_str
        add_callback( d, "in:" + piece_str, key )
        add_callback( d, "in:" + piece_str+"/", key )

    ##add_callback( d, "in:" + package, key )
    ##add_callback( d, "in:" + app, key )
    ##add_callback( d, "in:" + package+"/"+app, key )
  
    used_word = {}

    ###print profile ## may contain repetitions, but we get rid of them below

    for word in profile:
    ## sometimes we get empty string, no idea why
      assert word != ""

      if used_word.has_key(word):
        continue

      used_word[word] = 1
      add_callback( d, word, key )

    add_callback( d, "in:all", key )
      

def ensureTrailingSlash( s ):
  if s [ len(s) - 1 ] != '/':
    s = s + '/'
  return s

def writeDatabases():
  print
  print "*** writing databases"

  wm =  anydbm.open(cgidir+"word_messages.db", "n") # create new
  # functions include app
  wf =  anydbm.open(cgidir+"word_functions.db", "n") # create new

  mf =  anydbm.open(cgidir+"message_functions.db", "n") # create new
  fm =  anydbm.open(cgidir+"function_messages.db", "n") # create new

  fs = anydbm.open(cgidir+"function_source.db", "n") # create new
  ms = anydbm.open(cgidir+"message_source.db", "n") # create new
 

  ## clearly message should exclude application
  ## but, I think we need to keep a count of how many
  ## apps have that message, in another map

  ## clearly, ranking here can't depend on depth since
  ## all these GUI messages have a function with depth 0
  ## so instead it makes sense to rank them by # of apps
  ## say

  ## need to prefix GUI message with app
  for k in wm_mem.keys():  # depth 0
    ## need to add [0] after every msg
    msgs = wm_mem[k].split("\001")
    s = ""
    for m in msgs:
      if m != "":
        s = s + m + " [0]\000"
    s = s[:len(s)-1]
#    print "wm: " + k + " => " + s
    wm[k] = s
  wm.close()


  ## how should we rank functions?
  ## clearly, depth should be taken into account
  ## but what else?

  ## it's desirable to see standard callbacks
  ## appearing here
  ##
  ## the problem with counting # of apps is that
  ## it's unlikely that meaningful function names
  ## will occur unless of course you are overriding
  ## library functions

  ## 

  ## should function exclude application?
  ## probably, for consistency?

  ## word -> function [ best depth? ]
  for k in wf_mem.keys():  


    ## for each function, go through all its messages
    ## and pick best depth
    functions = wf_mem[k].split("\001")
#    print functions
    result = ""
    for f in functions:
      if f != "":
        #print f
        best_depth = 9999
        if fm_mem.has_key(f):
          msgs = fm_mem[f].split("\000")
#          print msgs
          for msg in msgs:
            if msg != "":
              # print "..." + msg
              depth = int(msg[msg.index("[")+1:msg.index("]")])
              if depth < best_depth:
                best_depth = depth
        result = result + f + " [" + ("%d"%best_depth)+"]\000"
    result = result[:len(result)-1]
    assert result != ""
    
    wf[k] = result
  wf.close()

  for k in mf_mem.keys():
    f = mf_mem[k]
    mf[k] = mf_mem[k]
  mf.close()

  for k in fm_mem .keys():
    fm[k] = fm_mem[k]
  fm.close()

  for k in fs_mem.keys():
    fs[k] = fs_mem[k]
  fs.close()

  for k in ms_mem.keys():
    ms[k] = ms_mem[k]
  ms.close()



############################################################################





print "*** GUIIndex (KDE 2.x module)"
if len(sys.argv) < 4:
  print "Usage:  guiindex.py appdir(s) cgidir kdelibsdir"
  print "Example:  guisearch.py koffice/* cgidir kdelibs"
  sys.exit()

#print len(sys.argv)
appdirs = []
for a in sys.argv[1:len(sys.argv)-2]:
  d = ensureTrailingSlash(a)
  if  os.access( d, os.F_OK ): # make sure this is a directory
    appdirs.append( d )

cgidir = ensureTrailingSlash(sys.argv[len(sys.argv)-2])
kdelibsdir = ensureTrailingSlash(sys.argv[len(sys.argv)-1])

#print ".......appdirs = " , appdirs
#print ".......cgidir = " + cgidir
#print ".......kdelibsdir = " + kdelibsdir

app_count = 0
for appdir in appdirs:
  app_count=app_count+1
  print "\n<<<< " + appdir + " >>>> (", app_count,"/",len(appdirs),")"


  # app\000function => list of parents
  function_parents = {}
  
  # function => 1
  all_functions = {}


  # app\000function depth
  is_callback = {}

  callback_message = {}

  # takes:  package / app : slot => ...
  slot_file = {}
  slot_lineno = {}



  ## called once per app
  kde.processKDEApplication( appdir, kdelibsdir, all_functions, function_parents, is_callback,ms_mem,callback_message,slot_file,slot_lineno )

  callback_message_keys = callback_message.keys()

  ####### compute distance of each function
  ## !! modifies callback_message by adding all other functions (that are not slots)

  function_callbacks = {}
  function_depth = {} # takes ( function, callback ) and gives depth with respect to callback

  compute_distances( function_parents, is_callback, function_callbacks, function_depth, callback_message )  

  global wordpart_cache
  wordpart_cache.clear() # clear for every app to reduce memory consumption
  print "*** cache size = ", len( wordpart_cache.keys() )
  ind = 0
  for function in all_functions.keys():
    ind = ind + 1
    print "*** processing function ", ind, " / ", len(all_functions.keys()), "\r",

    package = function[0:function.index("'")]

    ## this is where we extract profile from body
    body = ""
    str = ""
    ## for each file / lineno pair, pick up the relevant body
    pairs = zip( slot_file[function], slot_lineno[function] )
    for pair in pairs:
      delta = cpp.extract_body( appdir,pair[0], pair[1] )
      body = body + " " +  delta
      #      str = str + "<H3><A HREF=\"http://webcvs.kde.org/cgi-bin/cvsweb.cgi/"+pair[0]+"\">"+pair[0] + "</A>:" + ("%d"%pair[1]) + "</H3>\n<PRE>"+ delta +"\n</PRE>"
      str = str + "<H3>"+pair[0]+":"+ ("%d"%pair[1]) + "</H3>\n<PRE>"+ delta +"\n</PRE>"

      ## get src for all methods
    fs_mem[ function ] = str

    ## function body profile
    add_profile( function, extract_profile(body), wf_mem) 

    if is_callback[function]:
          assert callback_message.has_key(function)
          profile = [] ## no GUI message profile yet
          message = callback_message[function]
          message = misc.clean_msg(message)
          ## for GUI msg                       
          add_profile( package+"'"+message, extract_profile(message), wm_mem)     ## add under various GUI message names



    ## note:  callbacks can be associated with other callbacks
    for callback in function_callbacks[function]:
      message = callback_message[callback]
      message = misc.clean_msg(message)
      original_msg = message
      message = message + " [" + ("%d")%function_depth[function][callback] + "]"

      if fm_mem.has_key(function):
        fm_mem[function] = fm_mem[function] + "\000" + package+"'"+message
      else:
        fm_mem[function] = package+"'"+message

      ## inverse
      f2 = function + " [" + ("%d")%function_depth[function][callback] + "]"
      if mf_mem.has_key(package+"'"+original_msg):
        mf_mem[package+"'"+original_msg] = mf_mem[package+"'"+original_msg] + "\000" + f2
      else:
        mf_mem[package+"'"+original_msg] = f2 
  print

writeDatabases()



  
