## GUISearch:  Browse/Search Code of Applications written using a GUI Framework
## Copyright (C) 2001 Amir Michail

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version. 

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



import kde_xml
import re
import misc
import string


class callbacks:

  ################################# following vars should be cleared for each app
  # so create a new callbacks object for each app
  def __init__(self,appdir,callback_message,is_callback,stdaction_english_message,stdaction_english_heading,ms_mem):
    self.appdir = appdir
    self.callback_message = callback_message
    self.is_callback = is_callback
    self.app_internal_name_heading = {}
    self.stdaction_english_message = stdaction_english_message
    self.stdaction_english_heading = stdaction_english_heading
    self.ms_mem = ms_mem

    # looks for ) ) to end pattern
    self.find_answer_pattern = re.compile(r"\bSLOT\s*\(([^;]*?\))\s*\)", re.MULTILINE|re.DOTALL)
    self.find_query_pattern = re.compile(r"\bi18n\s*\(\s*\"([^\"]*?)\"", re.MULTILINE|re.DOTALL)

    try:
      ## hack (look for all rc files in this directory)
      print "*** loading app xml"
      kde_xml.load_xml( self.appdir, self.app_internal_name_heading )
    except IOError:
      print "*** WARNING:  couldn't find rc file"



  def add_slot(self,query,answer):
      message = query

      ## slot info
      ## get rid of parameters inside ()
      answer = string.strip(answer)
      answer = answer[0:answer.index("(")]
      answer = answer + "()"
      key = self.appdir+"'"+string.strip(answer)

  #    if self.callback_message.has_key(key):
  #      print "*** WARNING:  self.callback_message already has key " + key
      self.callback_message[key] = message

  #    print "  added callback " + key + " => " + message

      self.is_callback[key] = 1


  def parseCustomActions(self,original_text):
    ## general tip:  to avoid pattern going out of control, use [^;] instead of .

    ## also support KToggleAction
    find_action_pattern = re.compile(r"\bnew\s+K([a-zA-Z0-9_]*?)Action\b.*?;", re.MULTILINE|re.DOTALL)




    find_internal_name_pattern = re.compile(r"\"([^\"]*?)\"\s*\)\s*;", re.MULTILINE|re.DOTALL)


    text = original_text

    ################ handle custom actions
    while text != "":

      #    print "----------------------------------------"

      m = find_action_pattern.search(text)
      if ( m == None ):
        break

      action = m.group()
      #print action + "\n"
      text = text[m.end():] 

      # pick up string & slot
      m = self.find_query_pattern.search(action)
      if ( m == None):
  #      print "WARNING:  couldn't find query; skipping"
        continue
      query = m.group(1)

      # pick up slot to be called
      m = self.find_answer_pattern.search(action)
      if ( m == None):
  #      print "WARNING:  couldn't find answer; skipping"
        continue
      answer =  m.group(1)

      # pick up internal name
      m = find_internal_name_pattern.search(action)
      if ( m == None):
  #      print "WARNING:  couldn't find internal name; skipping"
        internal_name = "???"
      else:
        internal_name =  m.group(1)
        if self.app_internal_name_heading.has_key( internal_name ):
          query = self.app_internal_name_heading[internal_name] + " => " + query
  #      else:
  #        print "** warning couldn't find heading for -" + internal_name + "-"

  #    print self.appdir+ " KAction:  " + query + " => " + answer + " [" + internal_name + "]"

      self.ms_mem[ self.appdir + "'" + misc.clean_msg(query) ] = "<PRE>\n"+action+"\n</PRE>\n"

      ## should modify this to store multiple slots

      self.add_slot( query, answer)


  def parseStdActions(self,original_text):

    find_stdaction_pattern = re.compile(r"\bKStd[^;]*?Action\b.*?;", re.MULTILINE|re.DOTALL)

    find_action_type = re.compile(r"(^[^\s^(^;]*?)\s*\(", re.MULTILINE|re.DOTALL)

    text = original_text

    ################ handle standard actions
    while text != "":

  #    print "----------------------------------------"

      m = find_stdaction_pattern.search(text)
      if ( m == None ):
        break

      action = m.group()
  #    print action + "\n"
      text = text[m.end():] 

      # pick up slot to be called
      m = self.find_answer_pattern.search(action)
      if ( m == None):
  #      print "WARNING:  couldn't find answer; skipping"
        continue
      answer =  m.group(1)

      m = find_action_type.search(action)
      if ( m == None):
  #      print "WARNING:  couldn't find type; skipping"
        continue
      type =  m.group(1)

      try:
        query = self.stdaction_english_message[type]
      except KeyError:
  #      print "could not find action english message for " + type
        continue

      if self.stdaction_english_heading.has_key(type):
        query = self.stdaction_english_heading[type] + " => " + query

  #    print self.appdir + " " + type + " " + query + "=>" + answer


      self.ms_mem[ self.appdir + "'" + misc.clean_msg(query) ] = "<PRE>\n"+action+"\n</PRE>\n"

      ## should modify this to store multiple slots

      self.add_slot(query, answer)


  def parseInsertItem(self,original_text):
    ############## handle insertItem    

    ## this should match most insertItems; whether it contains SLOT or not
    ## should be checked in another pattern; this makes writing both patterns
    ## easier since we do not want to end up with one pattern that includes
    ## many insertItems just because some of them do not have SLOT say

    ## kind of tricky as there are several constructors and i18n may not come first
    ## general tip:  to avoid pattern going out of control, use [^;] instead of .
    find_insertitem_pattern = re.compile(r"\binsertItem\s*\([^;]*?i18n\s*\(.*?;", re.MULTILINE|re.DOTALL)

    text = original_text

    ################ handle custom actions
    while text != "":

    #    print "----------------------------------------"

      m = find_insertitem_pattern.search(text)
      if ( m == None ):
        break

      insertitem = m.group()
    #    print insertitem + "\n"
      text = text[m.end():] 

      # pick up string & slot
      m = self.find_query_pattern.search(insertitem)
      if ( m == None):
  #      print "WARNING:  couldn't find query; skipping"
        continue
      query = m.group(1)

      # pick up slot to be called
      m = self.find_answer_pattern.search(insertitem)
      if ( m == None):
  #      print "WARNING:  couldn't find answer; skipping"
        continue
      answer =  m.group(1)

  #    print self.appdir + " insertItem:  " + query + " => " + answer

      self.ms_mem[ self.appdir + "'" + misc.clean_msg(query) ] = "<PRE>\n"+insertitem+"\n</PRE>\n"

      self.add_slot(query, answer)



  ## called for every file
  def parse(self,file):

    f = open(file,"r")
    original_text = f.read()
    f.close()

    self.parseCustomActions(original_text)
    self.parseStdActions(original_text)
    self.parseInsertItem(original_text)











