#! /usr/bin/env python

## GUISearch:  Browse/Search Code of Applications written using a GUI Framework
## Copyright (C) 2001 Amir Michail

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version. 

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


import string
import anydbm
import re
import sys
import math

import search
import cpp

### ugly hack, need to remove qevent manually
### ideally these should not be generated as messages at all
def remove_qevent(final_list):
    final_list2 = []
    for f in final_list:
	if f.find("'qevent [") == -1:
	  final_list2.append(f)
    return final_list2

def fix_url(u):
    u2 = ""
    for i in u:
        if i == '=':
            u2 = u2 + "%3D"
        elif i == '>':
            u2 = u2 + "%3E"
        elif i == "'":
            u2 = u2 + "%27"
        elif i == " ":
            u2 = u2 + "+"
        else:
            u2 = u2 + i
    return u2

def shorten(s):
  if len(s) <= 32:
    return s;
  else:
    return s[0:32]+"..."  

def print_ranking( ranking, count, query, heading, component ):
    other = ""
    prefix = ""
    sprefix=""
    rprefix=""

    if heading == "Functions":
        other = "gui messages"
        if component:
            rprefix="ccmsgs:" # component combined messages
        else:
            rprefix="qcmsgs:" # query combined messages
        prefix = "msgs:"
        sprefix = "fsrc:"
    else:
        other = "functions"
        prefix = "fns:"
        if component:
            rprefix="ccfns:" # component combined functions
        else:
            rprefix="qcfns:" # query combined functions
        sprefix = "msrc:"

    print "<TABLE width=100%>"
    print "<TR class=s><TH colspan=3><font size=5>"+"Query \"" + query+"\" Resulted in " + ("%d"%count), # + " Matches</font>"
    print heading + " [ combined <A HREF=\"guisearch.cgi?query="+ rprefix  +  fix_url(query)  +"\">" + other + "</A> ]</font>"
    print "<TR class=s>"

    print "<TH>Application<TH>"+heading+"<TH>Depth"

    print "</TR>"
    depths = ranking.keys()
    depths.sort()

    prevapp = ""
    i = 0
    for d in depths:
        items = ranking[d]
        items.sort() # sort by package

        
        for item in items:
            it1 = item[0:item.index("'")]

            if it1 != prevapp:
                i = i + 1
                prevapp = it1
                
            if i % 2 == 1:
                print "<TR>"
            else:
                print "<TR class=o>"            
            
            it2 = item[item.index("'")+1:item.index(" [")]
            depth = int(item[item.index("[")+1:item.index("]")])
            it3 = item[0:item.index(" [")]

            print "<TD><B><TT>"+it1+"</TT></B><TD><B><TT>"+shorten(it2)+" [ <A HREF=\"guisearch.cgi?query="+sprefix+fix_url(it3)+"\">source</A> / <A HREF=\"guisearch.cgi?query="+prefix+ fix_url(it3) +"\">"+other+"</A> ]</TT></B>"
            if depth < 9999:
                 print "<TD align=center><B>"+("%d"%depth)+"</B></TR>"
            else:
                print "<TD align=center><B>?</B></TR>"
        
    print "</TR>"
    print "</TABLE>"


def add_profile( query, i, profile ):
    if not profile.has_key(query):
        profile[query] = []
    profile[query].append(i)

def show_combined( component,heading, db_fn, db_fn2, cmd_line_query ):

    query = merge_items( cmd_line_query )

    intermediate_list = []

    if component:

        d = anydbm.open(db_fn,"r")
        ans = d[query]

        intermediate_list = ans.split("\000")
        intermediate_list = remove_qevent(intermediate_list)
    else:
        searcher = search.Search(db_fn)
        intermediate_list = searcher.search( cmd_line_query ) # stem query words

    if intermediate_list == []:
        return

    d2 = anydbm.open(db_fn2, "r")


    profile = {} # takes message say & returns associated functions with their depths
    object_depth = {}

    item_list = []
    lowest_depth = {}
    for i in intermediate_list: # cycle through intermediate objects

        query_item_depth = int(i[i.index("[")+1:i.index("]")])
        i2 = i[0:i.index(" [")] # strip depth

        add_profile( query, i2, profile )

        object_depth[ query + "\000" + i2 ] = query_item_depth

        if not d2.has_key(i2):
            continue # some functions don't have an associated GUI message

        items = d2[i2]
        final_list = items.split("\000")
        final_list = remove_qevent(final_list)
        for f in final_list:
            other_item_depth = int(f[f.index("[")+1:f.index("]")])
            f2 = f[0:f.index(" [")]

            item_list.append(f2);

            if (not lowest_depth.has_key(f2)) or lowest_depth[f2] > other_item_depth:
                lowest_depth[f2] = other_item_depth
            add_profile( f2, i2, profile )
            object_depth[ f2 + "\000" + i2 ] = other_item_depth


    d2.close()

    ### uses minimum depth

    count = 0
    ranking = {}
    for k in item_list:
        v = lowest_depth[k]
        if not ranking.has_key(v):
            ranking[v] = []
        count = count + 1
        ranking[v].append(k+" ["+("%d"%lowest_depth[k])+"]")    

    print_ranking( ranking, count, query, heading, component )



def show_single( component,heading, db_fn, cmd_line_query ):

    query = merge_items( cmd_line_query )

    final_list = []

    if component:
        d = anydbm.open(db_fn,"r")
        ans = d[query]
        d.close()
        final_list = ans.split("\000")
    else:
        searcher = search.Search(db_fn)
        final_list = searcher.search( cmd_line_query ) # stem query words


    final_list = remove_qevent( final_list )

    if final_list == []:
        return

    ranking = {}

    ## sort first by rank
    for item in final_list:
        try:
          depth = int(item[item.index("[")+1:item.index("]")])
        except ValueError:
          # chokes on operator[]() [9999] so skip things like that
          continue
        if not ranking.has_key(depth):
            ranking[depth] = []
        ranking[depth].append(item)


    print_ranking( ranking, len(final_list), query, heading, component )



def merge_items(  items ):
    s = ""
    for i in items:
        if i != "":
            s = s + i + " "
    s = string.strip(s)
    return s


def show_message_source( msg ):
    app=msg[0:msg.index("'")]
    d = anydbm.open("message_source.db", "r")
    print cpp.hypertext(app,d[msg],"")
    d.close()

def show_function_source( msg ):
    app=msg[0:msg.index("'")]
    d = anydbm.open("function_source.db", "r")
    print cpp.hypertext(app,d[msg],msg)
    d.close()    

#######################################################

print """
<head>
<style type="text/css">
A:link, A:active, A:visited {color:#aaaaaa}
body {background-color:white;}
h1 {color:#0066cc; font-size:x-large}
.s {background-color:#3366CC; color:#FFFFFF;}
.o {background-color:#ccccee;}
.ref {color:#666666;font-size:12;font-family:"Arial, Helvetica, sans-serif"}

.popupLink { color: blue; outline: none;}
.popup { position:absolute; visibility:hidden; color:white;background-color:#3366cc}
layer-background-color:#3366cc;border:2px solid orange; padding: 3px; z-index: 10;}
</style>
</head>
<body bgcolor=white>
"""

query = sys.argv[1:]



## show source
if query[0].find("msrc:") == 0:
    show_message_source( merge_items( [query[0][5:]]+query[1:] ) )
elif  query[0].find("fsrc:") == 0:
    show_function_source( merge_items(  [query[0][5:]]+query[1:] ))

#### show_all shows combined functions/GUI messages for all components
#### show_single shows only functions/GUI messages for a single component/query

## show corresponding functions/GUI messages
elif query[0].find("fns:") == 0: ## parameter is a message
    show_single(1, "Functions",  "message_functions.db", ( [query[0][4:]]+query[1:]  )  )    # 1 for component
elif query[0].find( "msgs:") == 0:
    show_single(1, "GUI Messages",  "function_messages.db", (  [query[0][5:]]+query[1:]  ) ) # 1for component

## show combined functions/GUI messages
elif query[0].find("qcfns:") == 0: # query combined functions
    show_combined(0, "Functions",   "word_messages.db", "message_functions.db", ( [query[0][6:]]+query[1:] ) )    
elif query[0].find( "qcmsgs:") == 0: # query combined messages
    show_combined(0, "GUI Messages",  "word_functions.db",  "function_messages.db", ( [query[0][7:]]+query[1:]  ) )
elif query[0].find("ccfns:") == 0: # component combined functions
    show_combined(1, "Functions",   "function_messages.db", "message_functions.db", ( [query[0][6:]]+query[1:]  ) )    
elif query[0].find( "ccmsgs:") == 0: # component combined messages
    show_combined(1, "GUI Messages",  "message_functions.db",  "function_messages.db", ( [query[0][7:]]+query[1:] ) )        

## initial query case
else:
    show_single(0, "GUI Messages",  "word_messages.db", query ) # 0 for query
    show_single(0, "Functions", "word_functions.db", query ) # 0 for query
    print "End of results."



