#! /usr/bin/env python

# walk(path, visit, arg)

# Calls the function visit with arguments (arg, dirname, names) for each
# directory in the directory tree rooted at path (including path itself,
# if it is a directory). The argument dirname specifies the visited
# directory, the argument names lists the files in the directory (gotten
# from os.listdir(dirname)). The visit function may modify names to
# influence the set of directories visited below dirname, e.g., to avoid
# visiting certain parts of the tree. (The object referred to by names
# must be modified in place, using del or slice assignment.)

#
# adapted from os-path-walk-example-3.py
#
# obtained from usenet

import os
import fnmatch

class GlobDirectoryWalker:
    # a forward iterator that traverses a directory tree

    def __init__(self, directory, pattern="*"):
        self.stack = [directory]
        self.pattern = pattern
        self.files = []
        self.index = 0

    def __getitem__(self, index):
        while 1:
            try:
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # pop next directory from stack
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # got a filename
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                if fnmatch.fnmatch(file, self.pattern):
                    return fullname





